﻿/******************************************************************************
  * 文件名称: JsonHelper.cs
 * 文件说明: Json业务的封装，处理JSON串的封装，解析等
 * 当前版本: V1.0
 * 创建日期: 2022-04-14
 *
 * 2020-04-14: 增加 setIrisInpar 方法
 * 2020-04-14: 增加 setIrisInpar（重载） 方法
 * 2020-04-14: 增加 getIrisExceptionJson 方法
 * 2020-04-14: 增加 parseBusinessJson 方法

 ******************************************************************************/
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PTMedicalInsurance.Common;
using System.IO;
using PTMedicalInsurance.Variables;
using System.Windows.Forms;
using SM2Crypto.Lib;

namespace PTMedicalInsurance.Helper
{
    class JsonHelper
    {
        public static string toJsonString(object obj, bool missingNull)
        {
            JsonSerializerSettings settings = new JsonSerializerSettings();
            if (missingNull)
            {
                settings.NullValueHandling = NullValueHandling.Ignore;
            }
            return JsonConvert.SerializeObject(obj, Formatting.Indented, settings);
        }

        public static string toJsonString(object obj)
        {
            return toJsonString(obj, true);
        }

        /// <summary>
        /// 解析医保平台返回对象
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="response"></param>
        /// <returns></returns>
        public static T getOutput<T>(JObject response)
        {
            if (response.ContainsKey("output"))
            {
                return response["output"].ToObject<T>();
            }

            return response.ToObject<T>();
        }

        /// <summary>
        /// 根据JSonPath查找节点值，如果节点不存在则返回空值
        /// </summary>
        /// <param name="jo"></param>
        /// <param name="jsonPath"></param>
        /// <returns></returns>
        public static string getDestValue(JObject jo, string destPath)
        {
            JToken jt = jo.SelectToken("$." + destPath);
            if (jt != null)
            {
                return jt.ToString();
            }
            else
            {
                Global.writeLog(destPath + "的JToken属性值为空");
                return "";
            }
        }

        public static string getDestProperty(JObject jo, string propertyName)
        {
            JProperty jp = jo.Property(propertyName);
            if (jp != null)
            {
                string jpStr = jp.ToString();
                string rtnResult = "{ " + jpStr + "}";
                return rtnResult;
            }
            else
            {
                Global.writeLog(propertyName + "的JProperty属性值为空");
                return "";
            }
            
        }
        /// <summary>
        /// 压缩JSON，占用体积减小(重载)
        /// </summary>
        /// <param name="json"></param>
        /// <returns></returns>
        public static string Compress(string json)
  {
      StringBuilder sb = new StringBuilder();
      using (StringReader reader = new StringReader(json))
      {
          int ch = -1;
          int lastch = -1;
         bool isQuoteStart = false;
          while ((ch = reader.Read()) > -1)
         {
             if ((char) lastch != '\\' && (char) ch == '\"')
             {
                 if (!isQuoteStart)
                 {
                     isQuoteStart = true;
                 }
                 else
                 {
                     isQuoteStart = false;
                 }
             }
             if (!Char.IsWhiteSpace((char) ch) || isQuoteStart)
             {
                 sb.Append((char) ch);
             }
             lastch = ch;
         }
     }
     return sb.ToString();
 }
        public static string Compress(JObject jsonObj)
        {
            //https://blog.csdn.net/yangjiaosun/article/details/103717256
            return Newtonsoft.Json.JsonConvert.SerializeObject(jsonObj);
        }

        /// <summary>
        /// 组织异常JSON串
        /// </summary>
        /// <param name="errorCode"></param>
        /// <param name="errorMessage"></param>
        /// <returns></returns>
        public static JObject setExceptionJson(int errorCode, string eventName, string errorMessage)
        {
            dynamic joRtn = new JObject();
            joRtn.errorCode = errorCode;
            joRtn.errorMessage = eventName + "提示：" + errorMessage;
            return joRtn;
        }

        /// <summary>
        /// 组织中心入参
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar_ReadCard(string infno, JObject joInput)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;

            //异地卡读取1101的时候参保地置空
            if (Global.pat.bOtherProvLocal)
                Global.pat.insuplc_admdvs = "";

            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;
            Jo.mdtrtarea_admvs = "610199";   //陕西 市医保、省内异地、跨省异地就医地都传619900
            Global.pat.mdtrtarea_admvs = "610199";
            Jo.recer_sys_code = "XiAn";
            Jo.dev_no = Global.curEvt.mac;
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo;//
            Jo.signtype = "asn1"; ;
            Jo.infver = "v1.0";
            Jo.opter_type = Global.user.type; ;
            Jo.opter = Global.user.ID;
            Jo.opter_name = Global.user.name; ;
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;
            Jo.cainfo = "";
            Jo.Add("input", JObject.FromObject(joInput));

            return Jo.ToString();
        }

        /// <summary>
        /// 组织中心入参(明文)。部分代码是需要解析明文入参的，为了方便增加该方法
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar_plain(string infno, string input)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;

            Jo.mdtrtarea_admvs = "610199";   
            Global.pat.mdtrtarea_admvs = "610199";

            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;
            Jo.recer_sys_code = Global.inf.recivedSystem;
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo;//设备安全信息
            Jo.signtype = Global.inf.signatureType; ;
            Jo.infver = Global.inf.version; ;
            Jo.opter_type = Global.user.type; ;
            Jo.opter = Global.user.ID; ;
            Jo.opter_name = Global.user.name; ;
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;
            JObject joInput = (JObject)Newtonsoft.Json.JsonConvert.DeserializeObject(input);
            Jo.Add("input", JObject.FromObject(joInput));
            return Jo.ToString();
        }

        /// <summary>
        /// 组织中心入参(明文)。部分代码是需要解析明文入参的，为了方便增加该方法
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar_plain(string infno, JObject joInput)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;

            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;
            Jo.mdtrtarea_admvs = "610199";
            Global.pat.mdtrtarea_admvs = "610199";
            Jo.recer_sys_code = Global.inf.recivedSystem;
            Jo.dev_no = Global.inf.dev_no;
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo;//设备安全信息
            Jo.signtype = Global.inf.signatureType; ;
            Jo.infver = Global.inf.version; ;
            Jo.opter_type = Global.user.type; ;
            Jo.opter = Global.user.ID; ;
            Jo.opter_name = Global.user.name; ;
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;
            Jo.Add("input", joInput);
            return Jo.ToString();
        }

        /// <summary>
        /// 组织中心入参--签到
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar(string infno, string input)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;

            if (Global.inf.areaCode == "")
            {
                Global.inf.areaCode = "610199";
            }

            if (Global.pat.insuplc_admdvs == "")
            {
                Global.pat.insuplc_admdvs = Global.inf.areaCode;
            }          

            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;
            Jo.mdtrtarea_admvs = "610199"; 
            Global.pat.mdtrtarea_admvs = "610199";
            Jo.recer_sys_code = "XiAn";
            Jo.dev_no = Global.curEvt.mac;
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo ;//
            Jo.signtype = "SM3"; ;
            Jo.infver = "v1.0";
            Jo.opter_type = Global.user.type; ;
            Jo.opter = Global.user.ID; ;
            Jo.opter_name = Global.user.name; ;
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;

            if (input != "")
            {
                JObject joInput = (JObject)Newtonsoft.Json.JsonConvert.DeserializeObject(input);
                Jo.cainfo = "";
                Jo.Add("input", JObject.FromObject(joInput));
            }
            else
            {
                Jo.cainfo = "";
                Jo.Add("input", "");
            }

            return Jo.ToString();
        }

        /// <summary>
        /// 组织中心入参
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar(string infno, JObject joInput)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;

            if (Global.pat.insuplc_admdvs == "")
            {
                Global.pat.insuplc_admdvs = Global.inf.areaCode;      
            }            

            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;
            //Jo.mdtrtarea_admvs = "610100";   //陕西 市医保、省内异地、跨省异地就医地都传419900
            //Global.pat.mdtrtarea_admvs = "610100";

            if (Global.pat.mdtrtarea_admvs != "")
                Jo.mdtrtarea_admvs = "610199";
            else
                Jo.mdtrtarea_admvs = "610100";

            Jo.recer_sys_code = "XiAn";
            Jo.dev_no = Global.curEvt.mac; 
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo;//
            Jo.signtype = "asn1"; ;
            Jo.infver = "v1.0";
            Jo.opter_type = Global.user.type; ;
            Jo.opter = Global.user.ID; 
            Jo.opter_name = Global.user.name; ;
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;                      
            Jo.cainfo = "";

            Jo.Add("input", JObject.FromObject(joInput));

            return Jo.ToString();
        }

        /// <summary>
        /// 组织Iris入参
        /// </summary>
        /// <param name="code"></param>
        /// <param name="joParam"></param>
        /// <returns></returns>
        public static JObject setIrisInpar(string code, JObject joParam)
        {
            try
            {
                dynamic joInparam = new JObject();
                joInparam.code = code;
                dynamic joTmp = new JObject();
                JArray jaParam = new JArray();
                jaParam.Add(joParam);

                joInparam.Add("hospitalDr", Global.inf.hospitalDr);
                joInparam.Add("InterfaceDr", Global.inf.interfaceDr);

                joInparam.Add("params", JArray.FromObject(jaParam));
                joInparam.Add("session", Global.curEvt.jaSession);
                return joInparam;
            }
            catch (Exception ex)
            {
                return setExceptionJson(-1, "setIrisInpar:", ex.Message);
            }
        }

        /// <summary>
        /// 组织Iris入参
        /// </summary>
        /// <param name="code"></param>
        /// <param name="joParam"></param>
        /// <returns></returns>
        public static JObject setIrisInpar_(string code, JObject joParam,JObject joParam1)
        {
            try
            {
                JArray jaParam = new JArray();
                jaParam.Add(joParam);

                JArray jaParam1 = new JArray();
                jaParam1.Add(joParam1);

                dynamic joInparam = new JObject();
                joInparam.code = code;
                dynamic joTmp = new JObject();
          
                joInparam.Add("params", JArray.FromObject(jaParam));
                joInparam.Add("pagination", JArray.FromObject(jaParam1));
                joInparam.Add("session", Global.curEvt.jaSession);

                return joInparam;
            }
            catch (Exception ex)
            {
                return setExceptionJson(-1, "setIrisInpar_:", ex.Message);
            }
        }

        /// <summary>
        /// 组织Iris入参
        /// </summary>
        /// <param name="code"></param>
        /// <param name="joParam"></param>
        /// <returns></returns>
        public static JObject setIrisInpar(string code, JArray jaParams)
        {
            try
            {
                dynamic joInparam = new JObject();
                joInparam.code = code;
                dynamic joTmp = new JObject();

                //joInparam.Add("hospitalDr", Global.inf.hospitalDr);
                //joInparam.Add("InterfaceDr", Global.inf.interfaceDr);

                joInparam.Add("params", jaParams);
                joInparam.Add("session", Global.curEvt.jaSession);
                return joInparam;
            }
            catch (Exception ex)
            {
                return setExceptionJson(-1, "setIrisInpar:", ex.Message);
            }

        }

        /// <summary>
        /// 解析中心返参
        /// </summary>
        /// <param name="joRtn"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public static int parseCenterRtnValue(JObject joRtn,  out string errorMsg)
        {
            try
            {
                string errcode = getDestValue(joRtn, "errcode");
                string errorCode = getDestValue(joRtn, "errorCode");
                if (errcode != "")
                {
                    errorMsg = getDestValue(joRtn, "errmsg");
                    return -1;
                }
                else if (errorCode != "")
                {
                    errorMsg = getDestValue(joRtn, "errorMessage");
                    return -2;
                }
                else
                {
                    errorMsg = getDestValue(joRtn, "err_msg");
                    return int.Parse(getDestValue(joRtn, "infcode"));
                }
            }
            catch (Exception ex)
            {
                errorMsg = "解析中心返参发生异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 组织IRIS返参
        /// </summary>
        /// <param name="errorCode"></param>
        /// <param name="errorMessage"></param>
        /// <param name="joResult"></param>
        /// <returns></returns>
        public static JObject setIrisReturnValue(int errorCode, string errorMessage, JObject joResult)
        {
            try
            {
                dynamic joRtn = new JObject();
                joRtn.errorCode = errorCode;
                joRtn.errorMessage = errorMessage;
                joRtn.Add("result", joResult);
                return joRtn;
            }
            catch (Exception ex)
            {
                return setExceptionJson(-1, "setIrisReturnValue:", ex.Message);
            }
        }

        /// <summary>
        /// 解析IRIS返参
        /// </summary>
        /// <param name="joRtn"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public static int parseIrisRtnValue(JObject joRtn, out string errorMsg)
        {
            try
            {
                errorMsg = getDestValue(joRtn, "errorMessage");
                return int.Parse(getDestValue(joRtn, "errorCode"));
            }
            catch (Exception ex)
            {
                errorMsg = "解析Iris返参发生异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 解析业务JSON串（业务串固定格式为{errorCode = ,errorMessage = ,result ={}}）
        /// </summary>
        /// <param name="json"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public static int parseCenterReturnJson(JObject json, out string errorMsg)
        {
            try
            {
                errorMsg = "";
                if (int.Parse(json["infcode"].ToString()) != 0)
                {
                    errorMsg = json["err_msg"].ToString();
                }
                return int.Parse(json["infcode"].ToString());
            }
            catch (Exception ex)
            {
                errorMsg = "parseCenterReturnJson" + ex.Message;
                return -1;
            }

        }

        /// <summary>
        /// 移动支付组织中心入参
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setMPCenterInpar(string infno, string txtData)
        {
            EncryptHelper encrypt = new EncryptHelper();
            string plainText = SignUtil.SortInput(JObject.Parse(txtData));
            string signData = "";
            string output = encrypt.encrypt(txtData, ref signData);
            return output;
        }

        public static string setMPCenterInpar(string infno, JObject joInput)
        {
            return setMPCenterInpar(infno, joInput.ToString());
        }

        /// <summary>
        /// 解析移动返参
        /// </summary>
        /// <param name="joRtn"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public static int parseMPRtnValue(JObject joRtn, out string errorMsg)
        {
            try
            {
                string success = getDestValue(joRtn, "success");
                string message = getDestValue(joRtn, "message");
                string encData = getDestValue(joRtn, "encData");
                //Global.writeLog("success:" + success);
                if (success != "True")
                {
                    errorMsg = message;
                    return -1;
                }
                else
                {
                    errorMsg = joRtn.ToString();
                    return 0;
                }
            }
            catch (Exception ex)
            {
                errorMsg = "解析中心返参发生异常：" + ex.Message;
                return -1;
            }

        }

        /// <summary>
        /// 组织中心入参-医保电子处方流转
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInparPresCirNew(JObject data)
        {
            Global.writeLog("密钥id：" + Global.inf.PresCir.appid);
            Global.writeLog("密钥key：" + Global.inf.PresCir.secretKey);
            Global.writeLog("定点医药机构私钥：" + Global.inf.PresCir.privateKey);
            Global.writeLog("医保电子处方中心公钥：" + Global.inf.PresCir.publicKey);

            //Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            string timestamp = ((long)(DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).TotalMilliseconds).ToString();
            Global.curEvt.msgid = timestamp;

            JObject requestData = new JObject();
            requestData.Add("appId", Global.inf.PresCir.appid);
            //requestData.Add("version", "2.0.1");
            requestData.Add("encType", "SM4");
            requestData.Add("signType", "SM2");
            requestData.Add("timestamp", timestamp);

            //加密
            String encData = SMUtil.encrypt(JsonConvert.SerializeObject(data, Formatting.None), Global.inf.PresCir.appid, Global.inf.PresCir.secretKey);
            //Global.writeLog("入参加密前：" + "\r\n" + JsonConvert.DeserializeObject(data.ToString()) + "\r\n" + "入参加密后：" + "\r\n" + encData);

            JObject signDto = (JObject)JObject.Parse(JsonConvert.SerializeObject(requestData, Formatting.None));
            signDto.Add("data", data);

            //加签
            String signData = SMUtil.sign(signDto, Global.inf.PresCir.secretKey, Global.inf.PresCir.privateKey);
            Global.writeLog("签名结果：" + "\r\n" + signData);

            //报文
            requestData.Add("encData", encData);
            requestData.Add("signData", signData);

            dynamic Jo = new JObject();
            Jo.appId = Global.inf.PresCir.appid;
            //Jo.version = "2.0.1";
            Jo.timestamp = timestamp;
            Jo.encType = "SM4";
            Jo.signType = "SM2";
            Jo.signData = signDto.ToString();
            Jo.encData = data.ToString();

            Global.writeLog("加密前报文：" + "\r\n" + Jo.ToString() + "\r\n" + "加密后报文：" + "\r\n" + JsonConvert.SerializeObject(requestData, Formatting.None));

            //本地解密
            //String decData = SMUtil.decrypt(encData, Global.inf.appId, Global.inf.Secret);
            //Global.writeLog("解密：" + decData);
            //使用应用公钥（publicKey示例默认值）对本地自签名的数据进行验签示例
            //Boolean isVerify = SMUtil.verify(signDto, Global.inf.Secret, Global.inf.publicKey, signData);
            //Global.writeLog("验签：" + isVerify);

            return requestData.ToString();
        }

        /// <summary>
        /// 解析中心返参
        /// </summary>
        /// <param name="joRtn"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public static int parseCenterRtnValuePresCir(JObject joRtn, out string errorMsg)
        {
            try
            {
                string errcode = getDestValue(joRtn, "errcode");
                string errorCode = getDestValue(joRtn, "errorCode");
                if (errcode != "")
                {
                    errorMsg = getDestValue(joRtn, "message");
                    return -1;
                }
                else if (errorCode != "")
                {
                    errorMsg = getDestValue(joRtn, "errorMessage");
                    return -2;
                }
                else
                {
                    errorMsg = getDestValue(joRtn, "message");
                    return int.Parse(getDestValue(joRtn, "code"));
                }
            }
            catch (Exception ex)
            {
                errorMsg = "解析中心返参发生异常：" + ex.Message;
                return -1;
            }
        }
    }
}
